#ifndef __YLIB__
#define __YLIB__

#include <X11/Xlib.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>


///////////////////////////////////////////////////
// Les constantes et macros
///////////////////////////////////////////////////

/* Pour les imprcisions de calcul */
#define ABOUT_NUL 1e-10 // Valeur approche de zro
#define IS_ABOUT_NUL(x) (fabs(x)<ABOUT_NUL)

/* Les diffrents types de vue possibles */
#define V_PROJ 0 // Vue projete
#define V_PERS 1 // Vue en perspective



///////////////////////////////////////////////////
// Structures graphiques
///////////////////////////////////////////////////


typedef struct {
  double x,y,z;
} point_3d;

typedef struct {
  double x,y;
} point_2d;

/* Structures d'un objet 3d dans la fichier de modlisation */

typedef struct {
  short arete;
  double x,y,z;
} sommet_rec;

typedef struct {
  short deb;    // point de dpart
  short fin;    // point de fin
  short suiv_p; // arte suivante dans le sens positif
  short suiv_m; // arte suivante dans le sens ngatif
} arete_rec;

typedef struct {
  short arete; // une arte de la face
  short sens;  // <0:ngatif >0:positif
} face_rec;

/* Structures d'un objet 3d en mmoire */

typedef struct {
  double x,y,z;  // coordones du point
  short *aretes; // tab des artes squantes en ce sommet
  short *faces;  // tab des faces squantes en ce sommet
} sommet_3d;

typedef struct {
  short deb;      // Sommet de dpart
  short fin;      // Sommet d'arrive
  short face_p;   // Face sens positif
  short face_m;   // Face sens ngatif
  double a,b,c,d; // Coefficients de l'quation du plan passant par l'arte et  l'observateur
  char eq_valide; // Vaut 0 si les coeff ci-dessus doivent tre recalculs, 1 sinon
} arete_3d;

typedef struct {
  short *sommets; // Tableau des sommets de la face
  short nb_sommets;
  short *aretes;  // Tableau des artes de la face
  short nb_aretes;
  double a,b,c,d; // Coefficients de l'quation de la face
} face_3d;

typedef struct {
  arete_3d *aretes;
  short nb_aretes;
  face_3d *faces;
  short nb_faces;
  sommet_3d *sommets;
  short nb_sommets;
} objet_3d;



///////////////////////////////////////////////////
// Variables graphiques
///////////////////////////////////////////////////


/* Le contexte graphique */

extern GC       gc;         // Contexte graphique
extern Display *display;    // Connection serveur
extern Window   root;       // Fentre racine
extern Window   win;        // Fentre graphique (Texte 3D)
extern int      screen;     // Numro d'affichage de l'cran
extern Pixmap   buffer;     // Double buffer pour supprimer le scintillement
extern unsigned long pixel_blanc; // Valeur du blanc
extern unsigned long pixel_noir;  // Valeur du noir
extern unsigned long foreground;  // Couleur de l'avant-plan
extern unsigned long background;  // Couleur de l'arrire-plan
extern unsigned int  width;       // Largeur de la fentre graphique
extern unsigned int  height;      // Hauteur de la fentre graphique

/* La scne */

extern objet_3d *objets; // Liste des objets de la scne

#endif
